#include <iostream>

/*
 * This program implements an operator tree that is interpreted
 * via the common eval() interface that all operators share.
 * More specifically, this file contains implementations for literals
 * and operators of the boolean algebra ("and", "or", "not").
 * The common interface is eval which evaluates a subtree and returns
 * a boolean value.
 *
 * TODO: Implement the logical AND operator.
 */

#define MAXARGS 2

class SimpleOpBase {
  public:
    virtual bool eval () = 0;
    virtual ~SimpleOpBase () {}
  protected:
    SimpleOpBase* args [MAXARGS];
};


class And : public SimpleOpBase {
  public:
    // TODO: Constructor
    bool eval () {
      // TODO: logic
      return false;
    }
};


class Or : public SimpleOpBase {
  public:
    Or (SimpleOpBase* left, SimpleOpBase* right) : SimpleOpBase() {
      args[0] = left;
      args[1] = right;
    }
    bool eval () {
      return args[0]->eval() || args[1]->eval();
    }
};


class Not : public SimpleOpBase {
  public:
    Not (SimpleOpBase* op) : SimpleOpBase() {
      args[0] = op;
    }
    bool eval () {
      return !(args[0]->eval());
    }
};


class Literal : public SimpleOpBase {
  public:
    Literal (bool val) : _val(val) {}
    bool eval () {return _val;}
    bool getVal () {return _val;}
  private:
    bool _val;
};


class Implication : public SimpleOpBase {
  public:
    Implication (SimpleOpBase* left, SimpleOpBase* right) : SimpleOpBase() {
      args[0] = left;
      args[1] = right;
    }
    bool eval () {
      Not notLeft        (args[0]);
      Or  notLeftOrRight (&notLeft, args[1]);
      return notLeftOrRight.eval(); 
    }
};


int main () {
  Literal rains (false);
  Literal wetStreet (true);
  Implication implies (&rains, &wetStreet);

  std::cout << "\"If it "
            << (rains.getVal() ? "rains " : "does not rain ")
            << "then the street is "
            << (wetStreet.getVal() ? "is wet.\"" : "is not wet.\"")
            << " is a " << (implies.eval() ? "logical "  : "illogical ") << "implication"
            << std::endl;
}
